package rm;

import java.util.List;

/* Contains the instructions of a single program. */
public class Program {

	final List<Instruction> instructions;
	
	public final int maxRegisterId;
	
	public Program(List<Instruction> instructions, boolean allowErroneousHalts) {
		this.instructions = instructions;
		int curMaxRegId = -1;
		
		// Verify that all labels accessed are within
		// the instructions.
		for (Instruction instruction : this.instructions) {
			for (Label label : instruction.getAccessingLabels()) {
				if (!checkValidLabel(label, allowErroneousHalts)) {
					throw new IllegalArgumentException(
							"Invalid label " + label + " given at instruction :" + instruction);
				}
			}
			for (Integer regId : instruction.getAccessingRegisters()) {
				if (curMaxRegId < regId) {
					curMaxRegId = regId;
				}
			}
		}
		this.maxRegisterId = curMaxRegId;
	}
	
	private boolean checkValidLabel(Label label, boolean allowErroneousHalts) {
		if (label instanceof HaltLabel) return true;
		IntegerLabel intLabel = (IntegerLabel) label;
		return 0 <= intLabel.labelId && (allowErroneousHalts || intLabel.labelId < this.instructions.size());
	}
	
	@Override
	public String toString() {
		return instructions.toString();
	}
	
}
