package rm;

public class RegisterMachine {

	private long maxIter = Long.MAX_VALUE;
	private boolean shouldDebug = false;
	
	/* 
	 * Sets the maximum number of computation steps for the program.
	 * (This is not part of the formal definition).
	 */
	void setMaxIter(long maxIter) {
		this.maxIter = maxIter;
	}
	
	/* Sets whether to print the configuration at each step. */
	void setDebug(boolean shouldDebug) {
		this.shouldDebug = shouldDebug;
	}
	
	/* Executes a program starting from the given configuration. */
	public void execute(Program program, RegisterConfiguration config) {
		if (program.maxRegisterId >= config.getNumRegisters()) {
			throw new IllegalArgumentException("Register Machine requires " + program.maxRegisterId + " to run this program.");
		}
		long iter = 0;
		int labelId = 0;
		while (iter < maxIter) {
			++iter;
			if (this.shouldDebug) {
				System.out.println("(" + labelId + ", " + config + ")");
			}
			Label nextLabel = program.instructions.get(labelId).execute(config);
			if (nextLabel instanceof HaltLabel) {
				break;
			} else {
				labelId = ((IntegerLabel) nextLabel).labelId;
			}
		}
	}
	
}
