package rm;

import java.math.BigInteger;
import java.util.Arrays;
import java.util.List;

/* 
 * Instruction to subtracting one from a register and 
 * conditionally branching to a specified instruction
 * if the contents of the register is zero.
 */
public class SubOne implements Instruction {

	private final int regId;
	private final IntegerLabel posJump;
	private final IntegerLabel zeroJump;
	
	public SubOne(int regId, int posJump, int zeroJump) {
		this.regId = regId;
		this.posJump = new IntegerLabel(posJump);
		this.zeroJump = new IntegerLabel(zeroJump);
	}

	@Override
	public Label execute(RegisterConfiguration config) {
		BigInteger a = config.getRegister(this.regId);
		if (a.signum() > 0) {
			config.setRegister(regId, a.subtract(BigInteger.ONE));
			return posJump;
		} else {
			return zeroJump;
		}
	}

	@Override
	public List<Label> getAccessingLabels() {
		return Arrays.asList(posJump, zeroJump);
	}

	@Override
	public List<Integer> getAccessingRegisters() {
		return Arrays.asList(this.regId);
	}
	
	@Override
	public String toString() {
		return "R" + this.regId + "- -> L" + this.posJump + ", L" + this.zeroJump;
	}
	
}
